classdef Camera < handle
  % Camera
  %   The main interface to the depth cameras
  %
  % Methods:
  %   start - Start streaming of the camera
  %   stop - Stop streaming of the camera
  %   isStreaming - Check whether the camera is streaming
  %   isConnected - Check whether the camera is still connected
  %   getStreamList - Get the different output streams the camera can provide
  %   setStreamList - Set the streams that you want to get from the camera
  %   isStreamListSet - Check whether the stream list is set
  %   getSetStreamList - Get the stream list that is set on the camera
  %   getProcessConfig - Get the processing config of the camera
  %   setProcessConfig - Set the processing config of the camera
  %   hasFrames - Check if the camera has frames available
  %   getFrames - Get data frames from camera
  %   clearBuffer - Clear the circular buffer used to store frames
  %   getBufferSize - Get the circular buffer size
  %   setBufferSize - Set the circular buffer size
  %   getCameraConfig - Get the currently active camera config
  %   getSerial - Get the serial number of the current camera
  %   getCalibration - Get calibration from the camera
  %   setCalibration - Set calibration. If it is a csf camera, the calibration
  %                    will only be set in the pipeline. If it's a kea camera
  %                    the calibration will also be uploaded
  %   getUserPointerCapacity - Get the amount of user pointers that can be
  %                            stored
  %   setUserPointerCapacity - Set the amount of user pointers that can be
  %                            stored
  %   addUserPointer - Add a pointer that will be filled with the specified data
  %                    type. This method allows you to fill data straight into
  %                    your own data type without any additional copying.
  %   clearUserPointers - Clear all user pointers in memory
  %

  properties(Access=protected)
    mPtr = uint64(0);
  end

  methods(Access=public)
    function obj = Camera(varargin)
      % Camera
      %   Construct new camera class
      %

      if nargin == 1 && isa(varargin{1}, 'uint64')
        obj.mPtr = varargin{1};
      else
        error("Can't construct empty object");
      end
    end


    function delete(obj)
      if obj.mPtr ~= 0
        tof.mex('tof_camera_delete', obj.mPtr);
        obj.mPtr = 0;
      end
    end

    function ptr = get_object_ptr(obj)
      ptr = obj.mPtr;
    end

    function start(obj)
      % start
      %   Start streaming of the camera
      %

      tof.mex('tof_camera_start', obj.mPtr);
    end

    function stop(obj)
      % stop
      %   Stop streaming of the camera
      %

      tof.mex('tof_camera_stop', obj.mPtr);
    end

    function ret_val = isStreaming(obj)
      % isStreaming
      %   Check whether the camera is streaming
      %
      % Output:
      %   ret_val - logical - Is streaming
      %

      ret_val = tof.mex('tof_camera_is_streaming', obj.mPtr);
    end

    function ret_val = isConnected(obj)
      % isConnected
      %   Check whether the camera is still connected
      %
      % Output:
      %   ret_val - logical - Is connected
      %

      ret_val = tof.mex('tof_camera_is_connected', obj.mPtr);
    end

    function ret_val = getStreamList(obj)
      % getStreamList
      %   Get the different output streams the camera can provide
      %
      % Output:
      %   ret_val - vector[Stream] - The streams
      %

      ret_val = tof.mex('tof_camera_get_stream_list', obj.mPtr);
      ret_val = arrayfun(@(vec_obj)tof.Stream(vec_obj), ret_val);
    end

    function setStreamList(obj, streams)
      % setStreamList
      %   Set the streams that you want to get from the camera
      %
      % Inputs:
      %   streams - vector[Stream] - The wanted streams
      %

      tof.mex('tof_camera_set_stream_list', obj.mPtr, arrayfun(@(vec_obj)vec_obj.get_object_ptr(), streams));
    end

    function ret_val = isStreamListSet(obj)
      % isStreamListSet
      %   Check whether the stream list is set
      %
      % Output:
      %   ret_val - logical - Is stream list set
      %

      ret_val = tof.mex('tof_camera_is_stream_list_set', obj.mPtr);
    end

    function ret_val = getSetStreamList(obj)
      % getSetStreamList
      %   Get the stream list that is set on the camera
      %
      % Output:
      %   ret_val - vector[Stream] - The currently set streams
      %

      ret_val = tof.mex('tof_camera_get_set_stream_list', obj.mPtr);
      ret_val = arrayfun(@(vec_obj)tof.Stream(vec_obj), ret_val);
    end

    function ret_val = getProcessConfig(obj)
      % getProcessConfig
      %   Get the processing config of the camera
      %
      % Output:
      %   ret_val - ProcessingConfig - The processing config
      %

      ret_val = tof.mex('tof_camera_get_process_config', obj.mPtr);
      ret_val = tof.ProcessingConfig(ret_val);
    end

    function setProcessConfig(obj, config)
      % setProcessConfig
      %   Set the processing config of the camera
      %
      % Inputs:
      %   config - ProcessingConfig - The processing config
      %

      tof.mex('tof_camera_set_process_config', obj.mPtr, config.get_object_ptr());
    end

    function ret_val = hasFrames(obj)
      % hasFrames
      %   Check if the camera has frames available
      %
      % Output:
      %   ret_val - logical - Has frames available
      %

      ret_val = tof.mex('tof_camera_has_frames', obj.mPtr);
    end

    function ret_val = getFrames(obj)
      % getFrames
      %   Get data frames from camera
      %
      % Output:
      %   ret_val - vector[Data] - Data frames
      %

      ret_val = tof.mex('tof_camera_get_frames', obj.mPtr);
      ret_val = arrayfun(@(vec_obj)tof.Data(vec_obj), ret_val);
    end

    function clearBuffer(obj)
      % clearBuffer
      %   Clear the circular buffer used to store frames
      %

      tof.mex('tof_camera_clear_buffer', obj.mPtr);
    end

    function ret_val = getBufferSize(obj)
      % getBufferSize
      %   Get the circular buffer size
      %
      % Output:
      %   ret_val - numeric - The maximum number of sets of frames that is
      %                       stored before the oldest is overwritten
      %

      ret_val = tof.mex('tof_camera_get_buffer_size', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setBufferSize(obj, size)
      % setBufferSize
      %   Set the circular buffer size
      %
      % Inputs:
      %   size - numeric - The maximum number of sets of frames that is stored
      %                    before the oldest is overwritten
      %

      tof.mex('tof_camera_set_buffer_size', obj.mPtr, size);
    end

    function ret_val = getCameraConfig(obj)
      % getCameraConfig
      %   Get the currently active camera config
      %
      % Output:
      %   ret_val - CameraConfig - The camera config
      %

      ret_val = tof.mex('tof_camera_get_camera_config', obj.mPtr);
      ret_val = tof.CameraConfig(ret_val);
    end

    function ret_val = getSerial(obj)
      % getSerial
      %   Get the serial number of the current camera
      %
      % Output:
      %   ret_val - string - The serial
      %

      ret_val = tof.mex('tof_camera_get_serial', obj.mPtr);
    end

    function ret_val = getCalibration(obj)
      % getCalibration
      %   Get calibration from the camera
      %
      % Output:
      %   ret_val - Calibration - The calibration
      %

      ret_val = tof.mex('tof_camera_get_calibration', obj.mPtr);
      ret_val = tof.Calibration(ret_val);
    end

    function ret_val = getUserPointerCapacity(obj)
      % getUserPointerCapacity
      %   Get the amount of user pointers that can be stored
      %
      % Output:
      %   ret_val - numeric - The capacity
      %

      ret_val = tof.mex('tof_camera_get_user_pointer_capacity', obj.mPtr);
      ret_val = double(ret_val);
    end

    function setUserPointerCapacity(obj, capacity)
      % setUserPointerCapacity
      %   Set the amount of user pointers that can be stored
      %
      % Inputs:
      %   capacity - numeric - The capacity
      %

      tof.mex('tof_camera_set_user_pointer_capacity', obj.mPtr, capacity);
    end

    function clearUserPointers(obj)
      % clearUserPointers
      %   Clear all user pointers in memory
      %

      tof.mex('tof_camera_clear_user_pointers', obj.mPtr);
    end

  end
end
