import ctypes
from .__library__ import lib, get_error
from .stream import *
from .processing_config import *
from .data import *
from .camera_config import *
from .calibration import *


class Camera():
    """
    The main interface to the depth cameras
    """
    def __del__(self):
        if self._ptr:
            lib.tof_camera_delete(self._ptr)
            self._ptr = ctypes.c_void_p()


    def __init__(self, ptr = None):
        """
        Construct new camera class
        """
        self._ptr = ctypes.c_void_p()

        if ptr != None:
            self._ptr = ptr
        else:
            raise RuntimeError("Can't construct empty class")

    def start(self):
        """
        Start streaming of the camera
        """
        return lib.tof_camera_start(self._ptr, get_error())


    def stop(self):
        """
        Stop streaming of the camera
        """
        return lib.tof_camera_stop(self._ptr, get_error())


    def isStreaming(self):
        """
        Check whether the camera is streaming

        Returns
        ----------
        bool
            Is streaming
        """
        return lib.tof_camera_is_streaming(self._ptr, get_error())


    def isConnected(self):
        """
        Check whether the camera is still connected

        Returns
        ----------
        bool
            Is connected
        """
        return lib.tof_camera_is_connected(self._ptr, get_error())


    def getStreamList(self):
        """
        Get the different output streams the camera can provide

        Returns
        ----------
        List[Stream]
            The streams
        """
        size = lib.tof_camera_get_stream_list(self._ptr, ctypes.c_void_p(), 0, get_error())
        array = (ctypes.c_void_p * size)()
        lib.tof_camera_get_stream_list(self._ptr, array, size, get_error())
        return [Stream(ptr = item) for item in array]

    def setStreamList(self, streams):
        """
        Set the streams that you want to get from the camera

        Parameters
        ----------
        streams : List[Stream]
            The wanted streams
        """
        return lib.tof_camera_set_stream_list(self._ptr, (ctypes.c_void_p * len(streams))(*[item._ptr for item in streams]), len(streams), get_error())


    def isStreamListSet(self):
        """
        Check whether the stream list is set

        Returns
        ----------
        bool
            Is stream list set
        """
        return lib.tof_camera_is_stream_list_set(self._ptr, get_error())


    def getSetStreamList(self):
        """
        Get the stream list that is set on the camera

        Returns
        ----------
        List[Stream]
            The currently set streams
        """
        size = lib.tof_camera_get_set_stream_list(self._ptr, ctypes.c_void_p(), 0, get_error())
        array = (ctypes.c_void_p * size)()
        lib.tof_camera_get_set_stream_list(self._ptr, array, size, get_error())
        return [Stream(ptr = item) for item in array]

    def getProcessConfig(self):
        """
        Get the processing config of the camera

        Returns
        ----------
        ProcessingConfig
            The processing config
        """
        return ProcessingConfig(ptr = lib.tof_camera_get_process_config(self._ptr, get_error()))


    def setProcessConfig(self, config):
        """
        Set the processing config of the camera

        Parameters
        ----------
        config : ProcessingConfig
            The processing config
        """
        return lib.tof_camera_set_process_config(self._ptr, config._ptr, get_error())


    def hasFrames(self):
        """
        Check if the camera has frames available

        Returns
        ----------
        bool
            Has frames available
        """
        return lib.tof_camera_has_frames(self._ptr, get_error())


    def getFrames(self):
        """
        Get data frames from camera

        Returns
        ----------
        List[Data]
            Data frames
        """
        size = lib.tof_camera_get_frames(self._ptr, ctypes.c_void_p(), 0, get_error())
        array = (ctypes.c_void_p * size)()
        lib.tof_camera_get_frames(self._ptr, array, size, get_error())
        return [Data(ptr = item) for item in array]

    def clearBuffer(self):
        """
        Clear the circular buffer used to store frames
        """
        return lib.tof_camera_clear_buffer(self._ptr, get_error())


    def getBufferSize(self):
        """
        Get the circular buffer size

        Returns
        ----------
        int
            The maximum number of sets of frames that is stored before the oldest is
            overwritten
        """
        return lib.tof_camera_get_buffer_size(self._ptr, get_error())


    def setBufferSize(self, size):
        """
        Set the circular buffer size

        Parameters
        ----------
        size : int
            The maximum number of sets of frames that is stored before the oldest is
            overwritten
        """
        return lib.tof_camera_set_buffer_size(self._ptr, size, get_error())


    def getCameraConfig(self):
        """
        Get the currently active camera config

        Returns
        ----------
        CameraConfig
            The camera config
        """
        return CameraConfig(ptr = lib.tof_camera_get_camera_config(self._ptr, get_error()))


    def getSerial(self):
        """
        Get the serial number of the current camera

        Returns
        ----------
        str
            The serial
        """
        return lib.tof_camera_get_serial(self._ptr, get_error()).decode("utf-8")


    def getCalibration(self):
        """
        Get calibration from the camera

        Returns
        ----------
        Calibration
            The calibration
        """
        return Calibration(ptr = lib.tof_camera_get_calibration(self._ptr, get_error()))


    def getUserPointerCapacity(self):
        """
        Get the amount of user pointers that can be stored

        Returns
        ----------
        int
            The capacity
        """
        return lib.tof_camera_get_user_pointer_capacity(self._ptr, get_error())


    def setUserPointerCapacity(self, capacity):
        """
        Set the amount of user pointers that can be stored

        Parameters
        ----------
        capacity : int
            The capacity
        """
        return lib.tof_camera_set_user_pointer_capacity(self._ptr, capacity, get_error())


    def clearUserPointers(self):
        """
        Clear all user pointers in memory
        """
        return lib.tof_camera_clear_user_pointers(self._ptr, get_error())


def selectStreams(camera, frame_types):
    """
    Select the streams you want to get from the camera

    Parameters
    ----------
    camera : Camera
        A camera
    frame_types : List[FrameType]
        The frame types to select

    Returns
    ----------
    int
        The amount of selected streams
    """
    return lib.tof_select_streams(camera._ptr, (ctypes.c_int * len(frame_types))(*frame_types), len(frame_types), get_error())


def selectStreamsModFreq(camera, frame_types, mod_freq):
    """
    Select the streams you want to get from the camera with the specified modulation
    frequency

    Parameters
    ----------
    camera : Camera
        A camera
    frame_types : List[FrameType]
        The frame types to select
    mod_freq : float
        The modulation frequency to select

    Returns
    ----------
    int
        The amount of selected streams
    """
    return lib.tof_select_streams_mod_freq(camera._ptr, (ctypes.c_int * len(frame_types))(*frame_types), len(frame_types), mod_freq, get_error())


