
#include <assert.h>
#include <chronoptics/tof/csf_writer.h>
#include <chronoptics/tof/kea_camera.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define CHECK_ERROR(error)                           \
  if (error) {                                       \
    printf("ERROR: %s\n", tof_error_message(error)); \
    tof_error_delete(error);                         \
    return -1;                                       \
  }

// Capture from KeaCamera and write to a CSF File
int capture_raw_example(const char *configName, const char *serialNumber,
                        const char *captureName, const size_t nframes) {
  tof_error_t error = NULL;

  tof_processing_config_t config =
      tof_processing_config_new_from_disk(configName, &error);
  CHECK_ERROR(error);

  // Create new camera
  tof_kea_camera_t kea_cam = tof_kea_camera_new(config, serialNumber, &error);
  CHECK_ERROR(error);

  tof_processing_config_delete(config);

  tof_camera_t cam = (tof_camera_t)kea_cam;

  size_t size = tof_camera_get_stream_list(cam, NULL, 0, &error);
  CHECK_ERROR(error);

  tof_stream_t *streams = malloc(sizeof(void *) * size);

  // Now set the stream list
  size = tof_camera_get_stream_list(cam, streams, size, &error);
  CHECK_ERROR(error);

  // Set the ouput stream desired.
  size_t raw_count = 0;
  for (size_t n = 0; n < size; n++) {
    enum tof_frame_type type = tof_stream_frame_type(streams[n], &error);
    CHECK_ERROR(error);

    if (type == TOF_FRAME_TYPE_RAW) {
      if (n != raw_count) {
        tof_stream_delete(streams[raw_count]);
      }

      streams[raw_count] = streams[n];
      raw_count++;
    }
  }

  tof_camera_set_stream_list(cam, streams, raw_count, &error);
  CHECK_ERROR(error);

  // Remove last streams
  for (size_t i = 0; i < raw_count; i++) {
    tof_stream_delete(streams[i]);
  }

  tof_csf_writer_t writer =
      tof_create_csf_writer_camera(captureName, cam, &error);
  CHECK_ERROR(error);

  // Start the camera
  tof_camera_start(cam, &error);
  CHECK_ERROR(error);

  tof_data_t *frames = malloc(sizeof(void *) * raw_count);

  size_t captured_frames = 0;
  while (captured_frames < nframes) {
    // Get the frames and write
    size_t frame_size = tof_camera_get_frames(cam, frames, raw_count, &error);
    CHECK_ERROR(error);

    assert(frame_size == raw_count);

    for (size_t i = 0; i < frame_size; i++) {
      tof_csf_writer_write_frame(writer, frames[i], &error);
      CHECK_ERROR(error);

      tof_data_delete(frames[i]);
    }

    captured_frames++;
  }

  tof_camera_stop(cam, &error);
  CHECK_ERROR(error);

  tof_csf_writer_delete(writer);
  tof_kea_camera_delete(kea_cam);

  return 0;
}

int main(int argc, char **argv) {
  // Example on how to use the C interface to find a camera, connect and save
  // data to disk
  if (argc != 5) {
    printf("%s usage: ", argv[0]);
    printf(" filterConfig.json serial nframes out.csf\n");
    printf("Incorrect number of args!\n");
    return -1;
  }

  // Connect to a camera, and stream out raw frames and write to disk.
  return capture_raw_example(argv[1], argv[2], argv[4], atoi(argv[3]));
}